// apt_config_widgets.cc
//
//  Copyright 2000 Daniel Burrows

#include "apt_config_widgets.h"
#include <generic/apt.h>
#include <generic/config_signal.h>

using namespace std;

apt_bool_widget::apt_bool_widget(string _label, string _item, bool _default)
  :vs_checkbutton(_label, aptcfg->FindB(_item, _default)),
   item(_item), my_default(_default)
{
}

void apt_bool_widget::commit()
{
  // Setting an option causes it to be saved.
  if(aptcfg->ExistsUser(item) ||
     aptcfg->FindB(item, my_default)!=get_checked())
    aptcfg->Set(item, get_checked()?"true":"false");
}

apt_string_widget::apt_string_widget(string _item, string _default)
  :vs_editline("", aptcfg->Find(_item, _default.c_str())),
   item(_item), my_default(_default)
{
}

void apt_string_widget::commit()
{
  if(aptcfg->ExistsUser(item) ||
     aptcfg->Find(item, my_default.c_str())!=get_text())
    aptcfg->Set(item, get_text());
}

apt_radio_widget::apt_radio_widget(string _item,
				   const vector<string> &_choices,
				   string _default)
  :item(_item), choices(_choices), my_default(_default)
{
}

void apt_radio_widget::commit()
{
  string choice;

  if(selection_valid())
    {
      choice=choices[get_selected()];

      if(aptcfg->ExistsUser(item) ||
	 aptcfg->Find(item, my_default.c_str())!=choice)
	aptcfg->Set(item, choice);
    }
}
