// vs_util.cc

#include "vs_button.h"
#include "vs_center.h"
#include "vs_editline.h"
#include "vs_frame.h"
#include "vs_label.h"
#include "vs_pager.h"
#include "vs_util.h"
#include "vs_scrollbar.h"
#include "vs_table.h"
#include "vs_text_layout.h"

#include <config/colors.h>
#include <config/keybindings.h>

#include <sigc++/adaptors/bind.h>
#include <sigc++/functors/mem_fun.h>

#include <aptitude.h>

using namespace std;

vscreen_widget *vs_dialog_ok(vscreen_widget *w, slot0arg okslot, string label,
			     int attr)
{
  vs_center *center=new vs_center;

  vs_table *table=new vs_table;
  table->set_bg(attr);

  vs_button *okbutton=new vs_button(label);
  okbutton->set_bg(attr);

  okbutton->pressed.connect(sigc::mem_fun(*center, &vscreen_widget::destroy));
  if(okslot)
    okbutton->pressed.connect(*okslot);

  table->add_widget(w, 0, 0, 1, 1, true, true);
  table->add_widget(new vs_center(okbutton), 1, 0, 1, 1, false, false);

  vs_frame *frame=new vs_frame(table);
  frame->set_bg(attr);

  center->add_widget(frame);
  return center;
}

vscreen_widget *vs_dialog_ok(fragment *msg, slot0arg okslot, string label,
			     int attr, bool scrollbar)
{
  vscreen_widget *w;

  if(scrollbar)
    {
      vs_table *t=new vs_table;
      w=t;

      vs_text_layout *l=new vs_text_layout(msg);
      vs_scrollbar *s=new vs_scrollbar(vs_scrollbar::VERTICAL);

      l->set_bg(attr);
      s->set_bg(attr);

      t->add_widget(l, 0, 0, 1, 1, true, true);
      t->add_widget_opts(s, 0, 1, 1, 1,
			 vs_table::ALIGN_RIGHT,
			 vs_table::ALIGN_CENTER | vs_table::FILL);

      l->location_changed.connect(sigc::mem_fun(*s, &vs_scrollbar::set_slider));
      s->scrollbar_interaction.connect(sigc::mem_fun(*l, &vs_text_layout::scroll));
    }
  else
    w=new vs_text_layout(msg);

  w->set_bg(attr);

  return vs_dialog_ok(w, okslot, label, attr);
}

vscreen_widget *vs_dialog_ok(fragment *msg, slot0arg okslot, int attr, bool scrollbar)
{
  return vs_dialog_ok(msg, okslot, _("Ok"), attr, scrollbar);
}

vscreen_widget *vs_dialog_ok(fragment *msg, slot0arg okslot, bool scrollbar)
{
  return vs_dialog_ok(msg, okslot, get_color("DefaultWidgetBackground")|A_REVERSE, scrollbar);
}

vscreen_widget *vs_dialog_ok(string msg, slot0arg okslot,
			     int attr)
{
  vscreen_widget *l=new vs_label(msg);
  l->set_bg(attr);

  return vs_dialog_ok(l, okslot, _("Ok"), attr);
}

vscreen_widget *vs_dialog_ok(string msg, slot0arg okslot)
{
  return vs_dialog_ok(msg, okslot, get_color("DefaultWidgetBackground")|A_REVERSE);
}

vscreen_widget *vs_dialog_yesno(vscreen_widget *widget,
				slot0arg yesslot,
				string yeslabel,
				slot0arg noslot,
				string nolabel,
				int attr,
				bool deflt)
{
  vs_center *center=new vs_center;

  vs_table *table=new vs_table;
  table->set_bg(attr);

  vs_button *yesbutton=new vs_button(yeslabel);
  yesbutton->set_bg(attr);
  vs_button *nobutton=new vs_button(nolabel);
  nobutton->set_bg(attr);

  yesbutton->pressed.connect(sigc::mem_fun(*center, &vscreen_widget::destroy));
  nobutton->pressed.connect(sigc::mem_fun(*center, &vscreen_widget::destroy));

  if(yesslot)
    yesbutton->pressed.connect(*yesslot);
  if(noslot)
    nobutton->pressed.connect(*noslot);

  table->connect_key("Yes", &global_bindings, yesbutton->pressed.make_slot());
  table->connect_key("No", &global_bindings, nobutton->pressed.make_slot());
  table->connect_key("Cancel", &global_bindings, nobutton->pressed.make_slot());

  table->add_widget(widget, 0, 0, 1, 2, true, true);
  table->add_widget_opts(yesbutton, 1, 0, 1, 1, vs_table::SHRINK|vs_table::ALIGN_CENTER, 0);
  table->add_widget_opts(nobutton, 1, 1, 1, 1, vs_table::SHRINK|vs_table::ALIGN_CENTER, 0);

  widget->show();
  yesbutton->show();
  nobutton->show();

  if(deflt)
    table->focus_widget(yesbutton);
  else
    table->focus_widget(nobutton);

  vs_frame *frame=new vs_frame(table);
  frame->set_bg(attr);

  center->add_widget(frame);

  return center;
}

vscreen_widget *vs_dialog_yesno(string msg,
				slot0arg yesslot,
				string yeslabel,
				slot0arg noslot,
				string nolabel,
				int attr,
				bool deflt)
{
  vscreen_widget *txt=new vs_label(msg, attr);

  return vs_dialog_yesno(txt, yesslot, yeslabel, noslot, nolabel, attr, deflt);
}

vscreen_widget *vs_dialog_yesno(string msg,
				slot0arg yesslot,
				slot0arg noslot,
				int attr,
				bool deflt)
{
  return vs_dialog_yesno(msg, yesslot, _("Yes"), noslot, _("No"), attr, deflt);
}

vscreen_widget *vs_dialog_yesno(string msg,
				slot0arg yesslot,
				slot0arg noslot,
				bool deflt)
{
  return vs_dialog_yesno(msg,
			 yesslot,
			 noslot,
			 get_color("DefaultWidgetBackground")|A_REVERSE,
			 deflt);
}


vscreen_widget *vs_dialog_yesno(fragment *msg,
				slot0arg yesslot,
				slot0arg noslot,
				bool scrollbar,
				bool deflt)
{
  return vs_dialog_yesno(msg,
			 yesslot,
			 noslot,
			 get_color("DefaultWidgetBackground")|A_REVERSE,
			 scrollbar,
			 deflt);
}

vscreen_widget *vs_dialog_yesno(fragment *msg,
				slot0arg yesslot,
				slot0arg noslot,
				int attr,
				bool scrollbar,
				bool deflt)
{
  return vs_dialog_yesno(msg, yesslot, _("Yes"), noslot, _("No"), attr,
			 scrollbar, deflt);
}

vscreen_widget *vs_dialog_yesno(fragment *msg,
				slot0arg yesslot,
				std::string yeslabel,
				slot0arg noslot,
				std::string nolabel,
				int attr,
				bool scrollbar,
				bool deflt)
{
  vscreen_widget *w;

  if(scrollbar)
    {
      vs_table *t=new vs_table;
      w=t;

      vs_text_layout *l=new vs_text_layout(msg);
      vs_scrollbar *s=new vs_scrollbar(vs_scrollbar::VERTICAL);

      l->set_bg(attr);
      s->set_bg(attr);

      t->add_widget(l, 0, 0, 1, 1, true, true);
      t->add_widget_opts(s, 0, 1, 1, 1,
			 vs_table::ALIGN_RIGHT,
			 vs_table::ALIGN_CENTER | vs_table::FILL);

      l->location_changed.connect(sigc::mem_fun(*s, &vs_scrollbar::set_slider));
      s->scrollbar_interaction.connect(sigc::mem_fun(*l, &vs_text_layout::scroll));
    }
  else
    w=new vs_text_layout(msg);

  w->set_bg(attr);

  return vs_dialog_yesno(w, yesslot, yeslabel, noslot, nolabel, attr, deflt);
}

vscreen_widget *vs_dialog_fileview(string fn,
				   slot0arg okslot,
				   slotarg<sigc::slot1<void, vs_pager *> > searchslot,
				   int attr)
{
  vs_file_pager *p=new vs_file_pager(fn);
  vs_scrollbar *scrollbar=new vs_scrollbar(vs_scrollbar::VERTICAL, 0, 0);
  vs_table *t=new vs_table;
  vs_frame *f=new vs_frame(t);
  vs_center *c=new vs_center(f);

  vs_button *b=new vs_button("Ok");

  b->pressed.connect(sigc::mem_fun(*c, &vs_center::destroy));

  if(okslot)
    b->pressed.connect(*okslot);

  b->set_bg(attr);
  f->set_bg(attr);
  t->set_bg(attr);

  t->add_widget_opts(p, 0, 0, 1, 1,
		     vs_table::SHRINK | vs_table::ALIGN_CENTER,
		     vs_table::SHRINK | vs_table::ALIGN_CENTER);
  t->add_widget_opts(scrollbar, 0, 1, 1, 1,
		     vs_table::ALIGN_CENTER,
		     vs_table::EXPAND | vs_table::SHRINK | vs_table::ALIGN_CENTER);
  t->add_widget_opts(b, 1, 0, 1, 2,
		     vs_table::ALIGN_CENTER | vs_table::SHRINK,
		     vs_table::ALIGN_CENTER);

  t->connect_key("Confirm", &global_bindings, b->pressed.make_slot());
  p->line_changed.connect(sigc::mem_fun(*scrollbar, &vs_scrollbar::set_slider));
  p->do_line_signal();
  scrollbar->scrollbar_interaction.connect(sigc::mem_fun(*p, &vs_pager::scroll_page));

  if(searchslot)
    p->connect_key("Search", &global_bindings, sigc::bind((sigc::slot1<void, vs_pager *> &) searchslot, p));

  return c;
}

vscreen_widget *vs_dialog_fileview(string fn,
				   slot0arg okslot,
				   slotarg<sigc::slot1<void, vs_pager *> > searchslot)
{
  return vs_dialog_fileview(fn, okslot, searchslot,
			    get_color("DefaultWidgetBackground")|A_REVERSE);
}

static void do_dialog_string(string s,
			     sigc::slot0<void> realslot)
{
  realslot();
}

static void also_do_dialog_string(vs_editline *e,
				  sigc::slot1<void, string> thestrslot)
{
  e->add_to_history(e->get_text());
  thestrslot(e->get_text());
}

vscreen_widget *vs_dialog_string(string msg,
				 string deflt,
				 slotarg<sigc::slot1<void, string> > slot,
				 slotarg<sigc::slot0<void> > cancel_slot,
				 slotarg<sigc::slot1<void, string> > changed_slot,
				 vs_editline::history_list *history,
				 int attr)
{
  vs_table *t=new vs_table;
  vs_label *l=new vs_label(msg);
  vs_editline *e=new vs_editline(rootwin.getmaxx()-6, "", deflt, history);
  vs_button *bok=new vs_button(_("Ok")), *bcancel=new vs_button(_("Cancel"));
  vs_frame *f=new vs_frame(t);
  vs_center *c=new vs_center(f);

  l->set_bg(attr);
  f->set_bg(attr);
  t->set_bg(attr);
  bok->set_bg(attr);
  bcancel->set_bg(attr);

  t->add_widget(l, 0, 0, 1, 2);
  t->add_widget(e, 1, 0, 1, 2);
  t->add_widget_opts(bok, 2, 0, 1, 1,
		     vs_table::ALIGN_CENTER|vs_table::SHRINK,
		     vs_table::ALIGN_CENTER);
  t->add_widget_opts(bcancel, 2, 1, 1, 1,
		     vs_table::ALIGN_CENTER|vs_table::SHRINK,
		     vs_table::ALIGN_CENTER);

  e->entered.connect(sigc::bind(sigc::ptr_fun(do_dialog_string),
				bok->pressed.make_slot()));
  if(changed_slot)
    e->text_changed.connect(*changed_slot);

  t->connect_key_post("Cancel", &global_bindings,
		      bcancel->pressed.make_slot());

  bok->pressed.connect(sigc::mem_fun(*c, &vscreen_widget::destroy));
  if(slot)
    bok->pressed.connect(sigc::bind(sigc::ptr_fun(also_do_dialog_string),
				    e, *slot));

  bcancel->pressed.connect(sigc::mem_fun(*c, &vscreen_widget::destroy));
  if(cancel_slot)
    bcancel->pressed.connect(*cancel_slot);

  return c;
}

vscreen_widget *vs_dialog_string(string msg,
				 string deflt,
				 slotarg<sigc::slot1<void, string> > slot,
				 slotarg<sigc::slot0<void> > cancel_slot,
				 slotarg<sigc::slot1<void, string> > changed_slot,
				 vs_editline::history_list *history)
{
  return vs_dialog_string(msg,
			  deflt,
			  slot,
			  cancel_slot,
			  changed_slot,
			  history,
			  get_color("DefaultWidgetBackground")|A_REVERSE);
}
