// vs_util.h      -*-c++-*-
//
//  Copyright 2000 Daniel Burrows
//
//  Provides a bunch of utility functions to construct prefabricated
// widget trees (for instance, handy message boxes)

#ifndef VS_UTIL_H
#define VS_UTIL_H

#include "slotarg.h"
#include "vs_editline.h"

#include <string>

class fragment;
class vscreen_widget;
class vs_pager;

// Canned dialog-boxes:

/** Create a dialog box with a single button.
 *
 *  \param widget the widget to place above the button.
 *
 *  \param okslot the slot to be triggered when the button is pressed.
 *
 *  \param label the label of the button
 *
 *  \param attr the attributes to use for the background of the dialog
 *  box, defaults to reverse-video of DefaultWidgetBackground.
 */
vscreen_widget *vs_dialog_ok(vscreen_widget *widget,
			     slot0arg okslot, std::string label,
			     int attr);

vscreen_widget *vs_dialog_ok(fragment *msg, slot0arg okslot=NULL, bool scrollbar=false);
vscreen_widget *vs_dialog_ok(fragment *msg, slot0arg okslot, int attr, bool scrollbar=false);
vscreen_widget *vs_dialog_ok(fragment *msg, slot0arg okslot, std::string label,
			     int attr, bool scrollbar=false);

vscreen_widget *vs_dialog_ok(std::string msg, slot0arg okslot=NULL);
vscreen_widget *vs_dialog_ok(std::string msg, slot0arg okslot,
			     int attr);
vscreen_widget *vs_dialog_ok(std::string msg, slot0arg okslot, std::string label,
			     int attr);

/** Create a dialog box with two buttons, labelled "yes" and "no".
 *
 *  \param widget the widget to place above the buttons
 *
 *  \param yesslot the callback to be triggered when "yes" is selected
 *
 *  \param yeslabel the label of the "yes" button
 *
 *  \param noslot the callback to be triggered when "no" is selected
 *
 *  \param yeslabel the label of the "no" button
 *
 *  \param attr the attribute to use as the background of widgets
 *  created by this routine
 *
 *  \param deflt if \b true, the "yes" button will be selected by default;
 *  otherwise, the "no" button will be selected by default.
 */
vscreen_widget *vs_dialog_yesno(vscreen_widget *widget,
				slot0arg yesslot,
				std::string yeslabel,
				slot0arg noslot,
				std::string nolabel,
				int attr,
				bool deflt=true);

vscreen_widget *vs_dialog_yesno(fragment *msg,
				slot0arg yesslot,
				slot0arg noslot,
				bool scrollbar=false,
				bool deflt=true);
vscreen_widget *vs_dialog_yesno(fragment *msg,
				slot0arg yesslot,
				slot0arg noslot,
				int attr,
				bool scrollbar=false,
				bool deflt=true);
vscreen_widget *vs_dialog_yesno(fragment *msg,
				slot0arg yesslot,
				std::string yeslabel,
				slot0arg noslot,
				std::string nolabel,
				int attr,
				bool scrollbar=false,
				bool deflt=true);

vscreen_widget *vs_dialog_yesno(std::string msg,
				slot0arg yesslot,
				slot0arg noslot,
				bool deflt=true);
vscreen_widget *vs_dialog_yesno(std::string msg,
				slot0arg yesslot,
				slot0arg noslot,
				int attr,
				bool deflt=true);
vscreen_widget *vs_dialog_yesno(std::string msg,
				slot0arg yesslot,
				std::string yeslabel,
				slot0arg noslot,
				std::string nolabel,
				int attr,
				bool deflt=true);

// The "searchslot" is called when a search is being conducted.
vscreen_widget *vs_dialog_fileview(std::string fn,
				   slot0arg okslot=NULL,
				   slotarg<sigc::slot1<void, vs_pager *> > searchslot=NULL);
vscreen_widget *vs_dialog_fileview(std::string fn,
				   slot0arg okslot,
				   slotarg<sigc::slot1<void, vs_pager *> > searchslot,
				   int attr);

vscreen_widget *vs_dialog_string(std::string msg,
				 std::string deflt,
				 slotarg<sigc::slot1<void, std::string> > okslot,
				 slotarg<sigc::slot0<void> > cancel_slot,
				 slotarg<sigc::slot1<void, std::string> > changed_slot,
				 vs_editline::history_list *history,
				 int attr);
vscreen_widget *vs_dialog_string(std::string msg,
				 std::string deflt,
				 slotarg<sigc::slot1<void, std::string> > slot,
				 slotarg<sigc::slot0<void> > cancel_slot,
				 slotarg<sigc::slot1<void, std::string> > changed_slot,
				 vs_editline::history_list *history);

#endif
